"""JARVIS Tags Cog."""
import asyncio
import re

from jarvis_core.db import q
from jarvis_core.db.models import Setting, Tag
from naff import AutocompleteContext, Client, Extension, InteractionContext
from naff.models.discord.embed import EmbedField
from naff.models.discord.enums import Permissions
from naff.models.discord.modal import InputText, Modal, TextStyles
from naff.models.naff.application_commands import (
    OptionTypes,
    SlashCommand,
    slash_option,
)
from thefuzz import process

from jarvis.utils import build_embed

invites = re.compile(
    r"(?:https?://)?(?:www.)?(?:discord.(?:gg|io|me|li)|discord(?:app)?.com/invite)/([^\s/]+?)(?=\b)",  # noqa: E501
    flags=re.IGNORECASE,
)


class TagCog(Extension):

    tag = SlashCommand(name="tag", description="Create and manage custom tags")

    @tag.subcommand(sub_cmd_name="get", sub_cmd_description="Get a tag")
    @slash_option(
        name="name",
        description="Tag to get",
        autocomplete=True,
        opt_type=OptionTypes.STRING,
        required=True,
    )
    async def _get(self, ctx: InteractionContext, name: str) -> None:
        tag = await Tag.find_one(q(guild=ctx.guild.id, name=name))
        if not tag:
            await ctx.send(
                "Well this is awkward, looks like the tag was deleted just now", ephemeral=True
            )
            return

        await ctx.send(tag.content)

    @tag.subcommand(sub_cmd_name="create", sub_cmd_description="Create a tag")
    async def _create(self, ctx: InteractionContext) -> None:
        modal = Modal(
            title="Create a new tag!",
            components=[
                InputText(
                    label="Tag name",
                    placeholder="name",
                    style=TextStyles.SHORT,
                    custom_id="name",
                    max_length=40,
                ),
                InputText(
                    label="Content",
                    placeholder="Content to send here",
                    style=TextStyles.PARAGRAPH,
                    custom_id="content",
                    max_length=1000,
                ),
            ],
        )

        await ctx.send_modal(modal)
        try:
            response = await self.bot.wait_for_modal(modal, author=ctx.author.id, timeout=60 * 5)
            name = response.responses.get("name")
            content = response.responses.get("content")
        except asyncio.TimeoutError:
            return

        noinvite = await Setting.find_one(q(guild=ctx.guild.id, setting="noinvite"))

        if (invites.search(content) or invites.search(name)) and noinvite.value:
            await response.send(
                "Listen, don't use this to try and bypass the rules", ephemeral=True
            )
            return
        elif not content.strip() or not name.strip():
            await response.send("Content and name required", ephemeral=True)
            return

        tag = await Tag.find_one(q(guild=ctx.guild.id, name=name))
        if tag:
            await response.send("That tag already exists", ephemeral=True)
            return

        content = re.sub(r"\\?([@<])", r"\\\g<1>", content)

        tag = Tag(
            creator=ctx.author.id,
            name=name,
            content=content,
            guild=ctx.guild.id,
        )
        await tag.commit()

        embed = build_embed(
            title="Tag Created",
            description=f"{ctx.author.mention} created a new tag",
            fields=[EmbedField(name="Name", value=name), EmbedField(name="Content", value=content)],
        )

        embed.set_author(
            name=ctx.author.username + "#" + ctx.author.discriminator,
            icon_url=ctx.author.display_avatar.url,
        )

        await response.send(embeds=embed)

    @tag.subcommand(sub_cmd_name="edit", sub_cmd_description="Edit a tag")
    @slash_option(
        name="name",
        description="Tag name",
        opt_type=OptionTypes.STRING,
        autocomplete=True,
        required=True,
    )
    async def _edit(self, ctx: InteractionContext, name: str) -> None:
        tag = await Tag.find_one(q(guild=ctx.guild.id, name=name))
        if not tag:
            await ctx.send("Tag not found", ephemeral=True)
            return
        elif tag.creator != ctx.author.id and not (
            ctx.author.has_permission(Permissions.ADMINISTRATOR)
            or ctx.author.has_permission(Permissions.MANAGE_MESSAGES)
        ):
            await ctx.send("You didn't create this tag, ask the creator to edit it", ephemeral=True)
            return

        modal = Modal(
            title="Edit a tag!",
            components=[
                InputText(
                    label="Tag name",
                    value=tag.name,
                    style=TextStyles.SHORT,
                    custom_id="name",
                    max_length=40,
                ),
                InputText(
                    label="Content",
                    value=tag.content,
                    style=TextStyles.PARAGRAPH,
                    custom_id="content",
                    max_length=1000,
                ),
            ],
        )

        await ctx.send_modal(modal)
        try:
            response = await self.bot.wait_for_modal(modal, author=ctx.author.id, timeout=60 * 5)
            name = response.responses.get("name")
            content = response.responses.get("content")
        except asyncio.TimeoutError:
            return

        noinvite = await Setting.find_one(q(guild=ctx.guild.id, setting="noinvite"))

        if (invites.search(content) or invites.search(name)) and noinvite.value:
            await response.send(
                "Listen, don't use this to try and bypass the rules", ephemeral=True
            )
            return
        elif not content.strip() or not name.strip():
            await response.send("Content and name required", ephemeral=True)
            return

        tag.content = re.sub(r"\\?([@<])", r"\\\g<1>", content)
        tag.name = name

        await tag.commit()

        embed = build_embed(
            title="Tag Updated",
            description=f"{ctx.author.mention} updated a tag",
            fields=[
                EmbedField(name="Name", value=name),
                EmbedField(name="Content", value=tag.content),
            ],
        )

        embed.set_author(
            name=ctx.author.username + "#" + ctx.author.discriminator,
            icon_url=ctx.author.display_avatar.url,
        )

        await response.send(embeds=embed)

    @tag.subcommand(sub_cmd_name="delete", sub_cmd_description="Delete a tag")
    @slash_option(
        name="name",
        description="Tag name",
        opt_type=OptionTypes.STRING,
        required=True,
        autocomplete=True,
    )
    async def _delete(self, ctx: InteractionContext, name: str) -> None:
        tag = await Tag.find_one(q(guild=ctx.guild.id, name=name))
        if not tag:
            await ctx.send("Tag not found", ephemeral=True)
            return
        elif tag.creator != ctx.author.id and not (
            ctx.author.has_permission(Permissions.ADMINISTRATOR)
            or ctx.author.has_permission(Permissions.MANAGE_MESSAGES)
        ):
            await ctx.send(
                "You didn't create this tag, ask the creator to delete it", ephemeral=True
            )
            return

        await tag.delete()
        await ctx.send(f"Tag `{name}` deleted")

    @tag.subcommand(sub_cmd_name="info", sub_cmd_description="Get info on a tag")
    @slash_option(
        name="name",
        description="Tag name",
        opt_type=OptionTypes.STRING,
        required=True,
        autocomplete=True,
    )
    async def _info(self, ctx: InteractionContext, name: str) -> None:
        tag = await Tag.find_one(q(guild=ctx.guild.id, name=name))
        if not tag:
            await ctx.send("Tag not found", ephemeral=True)
            return

        username, discrim, url = None, None, None
        author = await self.bot.fetch_user(tag.creator)
        if author:
            username = author.username
            discrim = author.discriminator
            url = author.display_avatar.url

        ts = int(tag.created_at.timestamp())

        embed = build_embed(
            title="Tag Info",
            description=f"Here's the info on the tag `{name}`",
            fields=[
                EmbedField(name="Name", value=name),
                EmbedField(name="Content", value=tag.content),
                EmbedField(name="Created At", value=f"<t:{ts}:F>"),
            ],
        )

        embed.set_author(
            name=f"{username}#{discrim}" if username else "Unknown Author",
            icon_url=url,
        )

        await ctx.send(embeds=embed)

    @_get.autocomplete("name")
    @_edit.autocomplete("name")
    @_delete.autocomplete("name")
    @_info.autocomplete("name")
    async def _autocomplete(self, ctx: AutocompleteContext, name: str) -> None:
        tags = await Tag.find(q(guild=ctx.guild.id)).to_list(None)
        names = [tag.name for tag in tags]
        results = process.extract(name, names, limit=25)
        choices = [{"name": r[0], "value": r[0]} for r in results]
        await ctx.send(choices=choices)


def setup(bot: Client) -> None:
    """Add TagCog to JARVIS"""
    TagCog(bot)
